package net.wanji.utc.dt.task;

import net.wanji.databus.dto.PlanSectionDTO;
import net.wanji.databus.dto.SchemePhaseLightsDTO;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.utc.dt.cache.CrossInfoCache;
import net.wanji.utc.dt.cache.SignalDataCache;
import net.wanji.utc.dt.pojo.dtconvert.PhaseStageStatusPojo;
import net.wanji.utc.dt.pojo.dtconvert.RunningStatusPojo;
import net.wanji.utc.dt.service.StaticInfoService;
import net.wanji.utc.dt.service.impl.DTSignalStatusServiceImpl;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2023/08/16 13:50
 */
@Component
public class RealTimeTask {
    @Resource(name = "commonThreadPoolExecutor")
    ThreadPoolTaskExecutor commonThreadPoolExecutor;

    @Resource
    StaticInfoService staticInfoService;

    @Scheduled(initialDelay = 30 * 1000, fixedRate = 60 * 1000)
    public void getRealTimeLightStatus() throws Exception {
        Map<String, CrossInfoPO> crossInfoCache = CrossInfoCache.getCrossInfoCache();
        if (!crossInfoCache.isEmpty()) {
            for (Map.Entry<String, CrossInfoPO> entry : crossInfoCache.entrySet()) {
                String crossId = entry.getKey();
                commonThreadPoolExecutor.execute(() -> {
                    try {
                        List<String> lightsStatusInfoPojos = DTSignalStatusServiceImpl.getLightsStatusInfoPojos(crossId);
                        SignalDataCache.realTimeLightCache.put(crossId, lightsStatusInfoPojos);
                        List<RunningStatusPojo> runningStatusPojos = DTSignalStatusServiceImpl.getRunningStatusPojos(crossId);
                        SignalDataCache.realTimeRunStatusCache.put(crossId, runningStatusPojos);
                        List<PhaseStageStatusPojo> phaseStageStatus = DTSignalStatusServiceImpl.getPhaseStageStatus(crossId);
                        SignalDataCache.realTimePhaseStatusStageCache.put(crossId, phaseStageStatus);
                        // 静态数据
                        SchemePhaseLightsDTO schemePhaseLightsDTO = new SchemePhaseLightsDTO();
                        schemePhaseLightsDTO.setCrossId(crossId);
                        staticInfoService.schemePhaseLights(schemePhaseLightsDTO);
                        PlanSectionDTO planSectionDTO = new PlanSectionDTO();
                        planSectionDTO.setCrossId(crossId);
                        planSectionDTO.setPlanNo(-1);
                        staticInfoService.planSection(planSectionDTO);
                        staticInfoService.crossSchedules(crossId);
                    } catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                });
            }

        }

    }
}
